@@ -36,10 +36,17 @@ class RiemannProtobufMixin(object):
         """Encode a list of Tensor events with protobuf"""
 
         message = proto_pb2.Msg(
-            events=[self.encodeEvent(e) for e in events]
+            events=[self.encodeEvent(e) for e in events if e._type=='riemann']
         )
 
         return message.SerializeToString()
+
+    def decodeMessage(self, data):
+        """Decode a protobuf message into a list of Tensor events"""
+        message = proto_pb2.Msg()
+        message.ParseFromString(data)
+
+        return message
     
     def sendEvents(self, events):
         """Send a Tensor Event to Riemann"""
