@@ -1,7 +1,9 @@
 import time
+import sys
 import os
 import importlib
 import re
+import copy
 
 import yaml
 
@@ -20,7 +22,8 @@ class TensorService(service.Service):
     def __init__(self, config):
         self.running = 0
         self.sources = []
-        self.outputs = []
+        self.lastEvents = {}
+        self.outputs = {}
 
         self.evCache = {}
         self.critical = {}
@@ -30,11 +33,15 @@ class TensorService(service.Service):
 
         self.factory = None
         self.protocol = None
+        self.watchdog = None
 
         self.config = config
 
         both = lambda i1, i2, t: isinstance(i1, t) and isinstance(i2, t)
 
+        if os.path.exists('/var/lib/tensor'):
+            sys.path.append('/var/lib/tensor')
+
         if 'include_path' in config:
             ipath = config['include_path']
             if os.path.exists(ipath):
@@ -105,43 +112,66 @@ class TensorService(service.Service):
             cl = output['output'].split('.')[-1]                # class
             path = '.'.join(output['output'].split('.')[:-1])   # import path
 
-            # Import the module and get the object output we care about
-            outputObj = getattr(importlib.import_module(path), cl)
+            # Import the module and construct the output object
+            outputObj = getattr(
+                importlib.import_module(path), cl)(output, self)
 
-            self.outputs.append(outputObj(output, self))
+            name = output.get('name', None)
+
+            # Add the output to our routing hash
+            if name in self.outputs:
+                self.outputs[name].append(outputObj)
+            else:
+                self.outputs[name] = [outputObj]
 
-        for output in self.outputs:
             # connect the output
-            reactor.callLater(0, output.createClient)
+            reactor.callLater(0, outputObj.createClient)
+
+    def createSource(self, source):
+        # 
+        if source.get('path'):
+            path = source['path']
+            if path not in sys.path:
+                sys.path.append(path)
+
+        # Resolve the source
+        cl = source['source'].split('.')[-1]                # class
+        path = '.'.join(source['source'].split('.')[:-1])   # import path
+
+        # Import the module and get the object source we care about
+        sourceObj = getattr(importlib.import_module(path), cl)
+
+        if not ('debug' in source):
+            source['debug'] = self.debug
+
+        if not ('ttl' in source.keys()):
+            source['ttl'] = self.ttl
+
+        if not ('interval' in source.keys()):
+            source['interval'] = self.inter
+
+        return sourceObj(source, self.sendEvent, self)
+
+    def setupTriggers(self, source, sobj):
+        if source.get('critical'):
+            self.critical[sobj] = [
+                (re.compile(k), v) for k, v in source['critical'].items()
+            ]
+
+        if source.get('warning'):
+            self.warn[sobj] = [
+                (re.compile(k), v) for k, v in source['warning'].items()
+            ]
 
     def setupSources(self, config):
         """Sets up source objects from the given config"""
         sources = config.get('sources', [])
 
         for source in sources:
-            # Resolve the source
-            cl = source['source'].split('.')[-1]                # class
-            path = '.'.join(source['source'].split('.')[:-1])   # import path
+            src = self.createSource(source)
+            self.setupTriggers(source, src)
 
-            # Import the module and get the object source we care about
-            sourceObj = getattr(importlib.import_module(path), cl)
-
-            if not ('debug' in source):
-                source['debug'] = self.debug
-
-            if not ('ttl' in source.keys()):
-                source['ttl'] = self.ttl
-
-            if not ('interval' in source.keys()):
-                source['interval'] = self.inter
-
-            for k, v in source.get('critical', {}).items():
-                self.critical[re.compile(k)] = v
-                
-            for k, v in source.get('warning', {}).items():
-                self.warn[re.compile(k)] = v
-
-            self.sources.append(sourceObj(source, self.sendEvent, self))
+            self.sources.append(src)
 
     def _aggregateQueue(self, events):
         # Handle aggregation for each event
@@ -149,34 +179,57 @@ class TensorService(service.Service):
         for ev in events:
             if ev.aggregation:
                 id = ev.id()
-                if id in self.evCache:
-                    tDelta = ev.time - self.evCache[id].time
-                    m = ev.aggregation(
-                        self.evCache[id].metric, ev.metric, tDelta)
-                    queue.append(ev.copyWithMetric(m))
+                thisM = ev.metric
 
-                self.evCache[id] = ev
+                if id in self.evCache:
+                    lastM, lastTime = self.evCache[id]
+                    tDelta = ev.time - lastTime
+                    m = ev.aggregation(
+                        lastM, ev.metric, tDelta)
+                    if m:
+                        ev.metric = m
+                        queue.append(ev)
+
+                self.evCache[id] = (thisM, ev.time)
             else:
                 queue.append(ev)
 
         return queue
 
-    def setStates(self, events):
-        for ev in events:
+    def setStates(self, source, queue):
+        for ev in queue:
             if ev.state == 'ok':
-                for k, v in self.warn.items():
+                for k, v in self.warn.get(source, []):
                     if k.match(ev.service):
                         s = eval("service %s" % v, {'service': ev.metric})
                         if s:
                             ev.state = 'warning'
 
-                for k, v in self.critical.items():
+                for k, v in self.critical.get(source, []):
                     if k.match(ev.service):
                         s = eval("service %s" % v, {'service': ev.metric})
                         if s:
                             ev.state = 'critical'
 
-    def sendEvent(self, events):
+    def routeEvent(self, source, events):
+        routes = source.config.get('route', None)
+    
+        if not isinstance(routes, list):
+            routes = [routes]
+
+        for route in routes:
+            if self.debug:
+                log.msg("Sending events %s to %s" % (events, route))
+ 
+            if not route in self.outputs:
+                # Non existant route
+                log.msg('Could not route %s -> %s.' % (
+                    source.config['service'], route))
+            else:
+                for output in self.outputs[route]:
+                   reactor.callLater(0, output.eventsReceived, events)
+
+    def sendEvent(self, source, events):
         """Callback that all event sources call when they have a new event
         or list of events
         """
@@ -188,12 +241,19 @@ class TensorService(service.Service):
             events = [events]
     
         queue = self._aggregateQueue(events)
-        self.setStates(queue)
 
-        for output in self.outputs:
-            if self.debug:
-                log.msg("Sending events %s" % queue)
-            reactor.callLater(0, output.eventsReceived, queue)
+        if queue:
+            if (source in self.critical) or (source in self.warn):
+                self.setStates(source, queue)
+
+            self.routeEvent(source, queue)
+
+        queue = []
+
+        self.lastEvents[source] = time.time()
+
+    def _startSource(self, source):
+        source.startTimer()
 
     @defer.inlineCallbacks
     def startService(self):
@@ -201,6 +261,7 @@ class TensorService(service.Service):
 
         if self.debug:
             log.msg("Starting service")
+
         stagger = 0
         # Start sources internal timers
         for source in self.sources:
@@ -208,14 +269,61 @@ class TensorService(service.Service):
                 log.msg("Starting source " + source.config['service'])
             # Stagger source timers, or use per-source start_delay
             start_delay = float(source.config.get('start_delay', stagger))
-            reactor.callLater(start_delay, source.startTimer)
+            reactor.callLater(start_delay, self._startSource, source)
             stagger += self.stagger
 
+        reactor.callLater(stagger, self.startWatchdog)
         self.running = 1
  
+    def startWatchdog(self):
+        # Start source watchdog
+        self.watchdog = task.LoopingCall(self.sourceWatchdog)
+        self.watchdog.start(10)
+
+    def sourceWatchdog(self):
+        """Watchdog timer function. 
+
+        Recreates sources which have not generated events in 10*interval if
+        they have watchdog set to true in their configuration
+        """
+        for i, source in enumerate(self.sources):
+            if not source.config.get('watchdog', False):
+                continue 
+            sn = repr(source)
+            last = self.lastEvents.get(source, None)
+            if last:
+                try:
+                    if last < (time.time()-(source.inter*10)):
+                        log.msg("Trying to restart stale source %s: %ss" % (
+                            sn, int(time.time() - last)
+                        ))
+
+                        s = self.sources.pop(i)
+                        try:
+                            s.t.stop()
+                        except Exception, e:
+                            log.msg("Could not stop timer for %s: %s" % (
+                                sn, e))
+
+                        config = copy.deepcopy(s.config)
+
+                        del self.lastEvents[source]
+                        del s, source
+
+                        source = self.createSource(config)
+
+                        reactor.callLater(0, self._startSource, source)
+                except Exception, e:
+                    log.msg("Could not reset source %s: %s" % (
+                        sn, e))
+
     @defer.inlineCallbacks
     def stopService(self):
         self.running = 0
 
-        for output in self.outputs:
-            yield defer.maybeDeferred(output.stop)
+        if self.watchdog:
+            self.watchdog.stop()
+
+        for n, outputs in self.outputs.items():
+            for output in outputs:
+                yield defer.maybeDeferred(output.stop)
